import copy
import random


class PlnickaException(Exception):
    pass


class Plnicka():

    def __init__(self, pocet_sietok=8):
        """Automatický systém pre plnenie sieťok jablkami."""
        self.__nastav_pocet_sietok(pocet_sietok)
        self.__sietky = [list() for i in range(pocet_sietok)]
        self.__zasobnik = []
        self.__sklad = []
        self.__pocet_jablk_spat_do_zasobnika = 0
        self.__pocet_jablk_na_spracovanie = 0
        self.__pocet_sietok_do_skladu = 0
        self.__odpad = []
        self.__nastav_jablko(None)

    def __nastav_pocet_sietok(self, pocet_sietok):
        if not isinstance(pocet_sietok, int):
            raise PlnickaException('Chyba! Počet sietok musí byť celé číslo.')
        if pocet_sietok < 1:
            raise PlnickaException('Chyba! Počet sieťok musí byť kladný.')
        self.__pocet_sietok = pocet_sietok

    def __vrat_pocet_sietok(self):
        return self.__pocet_sietok

    pocet_sietok = property(__vrat_pocet_sietok, doc='Počet sieťok do ktorých je možné jablká umiestňovať. Sieťky sú číslované od 0.')

    def __nastav_sietky(self):
        self.__sietky = [list() for i in range(self.pocet_sietok)]

    def __vrat_sietky(self):
        return copy.deepcopy(self.__sietky)

    sietky = property(__vrat_sietky, doc='Zoznamy jabĺk ktoré sú umiestnené v jednotlivých sieťkach.')

    def __vrat_sklad(self):
        return copy.deepcopy(self.__sklad)

    def __nastav_sklad(self):
        self.__sklad = []

    sklad = property(__vrat_sklad, doc='Zoznamy jabĺk v sieťkach, ktoré sú umiestnené v sklade.')

    def __do_zasobnika(self, prvok):
        if not isinstance(prvok, int) or prvok <= 0:
            raise PlnickaException('Chyba! Pokus o vloženie neplatného prvku do zásobnika.')
        self.__zasobnik.append(prvok)
        self.__pocet_jablk_na_spracovanie += 1
        max_recyklacia = 2
        if self.pocet_jablk_spat_do_zasobnika > max_recyklacia * self.pocet_jablk_na_spracovanie:
            raise PlnickaException(
                f'Chyba! Vyzerá to tak, že táto stratégia nie je veľmi úspešná. Každé jablko sa do zásobníka vrátilo priemerne viac ako {max_recyklacia}x. Ak sa to stáva často, pouvažujte nad inou stratégiou.')

    def _do_zasobnika(self, kontajner):
        """Naplní zásobník jablkami zo zadaného kontajnera. Kontajner je typu zoznam.

        príklad: plnicka.do_zasobnika([100, 120])
        """
        if not isinstance(kontajner, list):
            raise PlnickaException('Chyba! Obsah kontajnera sa nedá vložiť do zásobnika.')
        for prvok in kontajner:
            self.__do_zasobnika(prvok)
        kontajner.clear()

    def vyber_jablko_zo_zasobnika(self):
        """Vyberie jablko zo zásobníka na spracovanie."""
        if self.__jablko is not None:
            raise PlnickaException('Chyba! Nie je spracované predchádzajúce jablko.')
        try:
            self.__nastav_jablko(self.__zasobnik.pop(0))
        except IndexError:
            raise PlnickaException('Chyba! Zásobnik je prázdny.')

    def __nastav_jablko(self, jablko):
        self.__jablko = jablko

    def __vrat_jablko(self):
        if self.__jablko is None:
            raise PlnickaException('Chyba! Nie je spracovávané žiadne jablko.')
        return self.__jablko

    jablko = property(__vrat_jablko, doc='Hmotnosť spracovávaného jablka')

    def __je_prazdny_zasobnik(self):
        return len(self.__zasobnik) == 0

    je_prazdny_zasobnik = property(__je_prazdny_zasobnik, doc='Odpoveď na otázku: Je zásobník prázdny?')

    def jablko_do_sietky(self, sietka):
        """Premiestni spracovávané jablko do sieťky s uvedeným číslom.

        príklad: plnicka.jablko_do_sietky(0)
        """
        if self.jablko is None:
            raise PlnickaException('Chyba! Nie je vybrané jablko.')
        try:
            self.__sietky[sietka].append(self.jablko)
            self.__nastav_jablko(None)
        except (IndexError, TypeError):
            raise PlnickaException('Chyba! Vkladáte jablko do neexistujúcej sieťky.')

    def sietka_do_skladu(self, sietka):
        """Premiestni uvedenú sieťku s jablkami do skladu a na jej miesto vloží prázdnu sieťku"""
        try:
            self.__sklad.append(self.__sietky[sietka].copy())
            self.__sietky[sietka].clear()
            self.__pocet_sietok_do_skladu += 1
        except (IndexError, TypeError):
            raise PlnickaException('Chyba! Presúvate do skladu neexistujúcu sietku.')

    def __vrat_pocet_sietok_do_skladu(self):
        return self.__pocet_sietok_do_skladu

    pocet_sietok_do_skladu = property(__vrat_pocet_sietok_do_skladu, doc='Počet sieťok, ktoré boli premiestnené do skladu.')

    def sietka_do_zasobnika(self, sietka):
        """Premiestni jablká v sieťke do zásobníka na opätovné spracovanie a na jej miesto vloží prázdnu sieťku."""
        try:
            self.__pocet_jablk_spat_do_zasobnika += len(self.__sietky[sietka])
            random.shuffle(self.__sietky[sietka])
            self.__pocet_jablk_na_spracovanie -= len(self.__sietky[sietka])
            self._do_zasobnika(self.__sietky[sietka])
        except (IndexError, TypeError):
            raise PlnickaException('Chyba! Presúvate do zásobníka neexistujúcu sieťku.')

    def __vrat_pocet_jablk_spat_do_zasobnika(self):
        return self.__pocet_jablk_spat_do_zasobnika

    pocet_jablk_spat_do_zasobnika = property(__vrat_pocet_jablk_spat_do_zasobnika, doc='Počet jabĺk, ktoré boli vysypané späť do zásobníka.')

    def __vrat_pocet_jablk_na_spracovanie(self):
        return self.__pocet_jablk_na_spracovanie

    pocet_jablk_na_spracovanie = property(__vrat_pocet_jablk_na_spracovanie, doc='Počet jabĺk ktoré boli nasypané do zásobníka plničky na spracovanie. Jablká z vrátených sieťok sa nezapočítavajú.')

    def hmotnost_sietky(self, sietka):
        """Hmotnosť jabĺk v zadanej sieťke."""
        try:
            return sum(self.__sietky[sietka])
        except (IndexError, TypeError):
            raise PlnickaException('Chyba! Chcete odvážiť neexistujúcu sieťku.')

    def jablko_do_odpadu(self):
        """Premiestni spracovávané jablko do odpadu"""
        if self.jablko is None:
            raise PlnickaException('Chyba! Nie je vybrané jablko.')
        self.__odpad.append(self.jablko)
        self.__nastav_jablko(None)

    def __vrat_odpad(self):
        return self.__odpad.copy()

    odpad = property(__vrat_odpad, doc='Zoznam jabĺk v odpade.')

    def napln_zasobnik(self):
        """Naplní zásobník jablkami."""
        m = 150
        m_plus = 0.3
        m_minus = 0.3
        self._do_zasobnika([int(m + random.randint(0, m * (m_plus + m_minus)) - m * m_minus) for i in range(random.randint(90000, 100000))])

    def _hmotnost_jablk(self):
        spolu = sum(self.odpad)
        for sietka in range(self.pocet_sietok):
            spolu += self.hmotnost_sietky(sietka)
        for sietka in self.sklad:
            spolu += sum(sietka)
        spolu += sum(self.__zasobnik)
        return (spolu)

    def _test_sklad(self):
        chyba = True
        for sietka in self.sklad:
            if not (1000 * 0.95 <= sum(sietka) <= 1000 * 1.05):
                chyba = False
        return chyba

################################################################################
'''Naprogramuj funkciu spusti_plnicku()
'''
